<?php

use Illuminate\Database\Capsule\Manager as Capsule;

define('API_PRODUCTION', 'https://api.synergywholesale.com');
define('MODULE_VERSION', '1.9.12');

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_getConfigArray($params)
{
    // CURL to include URL to show the reseller IP address required to be added
    $configarray = [
        "FriendlyName"    => ['Value' => 'Synergy Wholesale Domains Module', 'Type' => 'System'],
        "resellerID"      => ["FriendlyName" => "Reseller ID", "Type" => "text", "Size" => "15", "Description" => "Enter your Reseller ID here"],
        "apiKey"          => ["FriendlyName" => "API Key", "Type" => "text", "Size" => "45", "Description" => "Enter your API Key here"],
        "doRenewal"       => ["FriendlyName" => "Force .AU Renewal on Transfer", "Type" => "yesno", "Size" => "1", "Description" => "Tick if you wish to perform a renewal on any .au domains submitted for transfer that are within 90 days of expiry."],
        "apiConnectivity" => ["FriendlyName" => "API Connectivity", "Type" => "yesno", 'Description' => 'Enable to see connectivity status to the Synergy Wholesale API'],
        "convertVIP"      => ["FriendlyName" => "Convert from old VentraIP Wholesale Module", "Type" => "yesno", 'Description' => 'Enable this option to convert from the old VentraIP Domains Module to this new Synergy Module - NOTE: This option will be disabled automatically again once you have clicked \'Save Changes\' and the conversion sequence is completed.'],
        "whoisUpdate"      => ["FriendlyName" => "Force update WHOIS.json", "Type" => "yesno", 'Description' => 'Enable this option to force update the WHOIS.json data - NOTE: This option will be disabled automatically again once you have clicked \'Save Changes\' and the update sequence is completed.'],        
        "Version"         => ["Description" => "This module version: " . MODULE_VERSION],
    ];

    if (isset($params['apiConnectivity']) && $params['apiConnectivity'] === 'on') {

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, "https://manage.synergywholesale.com/ip");
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $data = curl_exec($ch);
        curl_close($ch);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, API_PRODUCTION);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_exec($ch);

        if (!curl_errno($ch)) {
            $info                  = curl_getinfo($ch);
            $API_Production_status = $info['http_code'];
        } else {
            $API_Production_status = false;
        }

        curl_close($ch);

        $ipaddress = $data;

        switch ($API_Production_status) {
            case 200:
                $message     = '<b style="color:#1EB600">200 - Successful</b>';
                $testCommand = synergywholesaledomains_APICall('balance', [], $params);
                break;

            case 403:
                $message = '<b style="color:#FF0000;">403 - Access Denied</b>';
                break;

            case false:
                $message = '<b style="color:#FF0000;">Unable to connect: <i>Check firewall, or submit <a style="color:#FF0000;text-decoration:underline" target="_blank" href="https://manage.synergywholesale.com/home/support/new">Support Request</a></i></b>';
                break;
        }

        $whitelistedHtml = sprintf('Disable to hide connectivity status to the Synergy Wholesale API<br /><i>This should be disabled unless configuring</i><br /><br />Hosting IP Address: <b>%s</b>%s', $ipaddress, sprintf("<br /><i>You'll need to whitelist this IP address for the API usage within <a target='_blank' href='https://manage.synergywholesale.com/home/resellers/api'>Synergy Wholesale > API Information</a></i><br /><br />
            Production API Whitelisting: %s<br />Production API Authentication: %s",
            $message,
            ((isset($testCommand)) ? ((isset($testCommand['status']) && $testCommand['status'] === 'OK') ? '<b style="color:#1EB600">OK</b>' : '<b style="color:#FF0000;">' . $testCommand['status'] . '</b>') : 'N/A')
        ));

        $configarray['apiConnectivity']['Description'] = $whitelistedHtml;
    }

    // Check to see if we are configured to connect, then attempt to retrieve the latest version
    if ((isset($params['apiKey']) && strlen($params['apiKey']) > 0) && (isset($params['resellerID']) && strlen($params['resellerID']) > 0)) {

        // Chances are we can get the info we need
        $ret = synergywholesaledomains_APICall("getWHMCSModulesVersions", [], $params);
        if (!isset($ret['error'])) {
            foreach ($ret as $key => $value) {
                if ($value->key == "domains") {
                    if ($value->moduleVersion <= MODULE_VERSION) {
                        // No need to display new version, we have the latest version
                        $configarray['Version']['Description'] = $configarray['Version']['Description'] . "<br>You are currently running the latest module version available";
                    } else {
                        // Display new module data
                        $configarray['Version']['Description'] = $configarray['Version']['Description'] . "<br>Latest Available Version: " . $value->moduleVersion . "<br>Download latest version from <a href=\"https://synergywholesale.com/faq/api-documentation-whmcs-modules\" target=\"_blank\">here</a>";
                    }
                }
            }
        }
    }

    // If the conversion option is ticked, then we need to process the conversion
    if (isset($params['convertVIP']) && $params['convertVIP'] === 'on') {

        /*
        We need to convert database entries from ventraip
        to synergywholesaledomains
         */

        try {

            // Update any existing domain entries in tbldomains to synergywholesaledomains from ventraip
            Capsule::table('tbldomains')->where('registrar', 'ventraip')->update(['registrar' => 'synergywholesaledomains']);
            // Update any domain pricing for autoreg value to be synergywholesaledomains not ventraip
            Capsule::table('tbldomainpricing')->where('autoreg', 'ventraip')->update(['autoreg' => 'synergywholesaledomains']);
        } catch (\Exception $e) {
            // Unable to run the queries
            $configarray['convertVIP']['Description'] = $configarray['convertVIP']['Description'] . "<br><b>WARNING:</b> We were unable to automatically convert all of the domains to the new module. Please contact Synergy Wholesale support for further assistance.";
            return $configarray;
        }

        try {
            // Finally disable the setting for the customer automatically so
            // we don't have them needing to turn if off manually
            Capsule::table('tblregistrars')->where(['registrar' => 'synergywholesaledomains', 'setting' => 'convertVIP'])->update(['value' => 'off']);
        } catch (\Exception $e) {
            // Update the description so that the client knows something oddball happened
            $configarray['convertVIP']['Description'] = $configarray['convertVIP']['Description'] . "<br><b>NOTICE:</b> All entries successfully converted however we were unable to disable this option automatically for you. Please untick the option yourself manually and click 'Save Changes' again.";
            return $configarray;
        }

    }

        // If the conversion option is ticked, then we need to process the conversion
    if (isset($params['whoisUpdate']) && $params['whoisUpdate'] === 'on') {
        $jsonPath = [$_SERVER['DOCUMENT_ROOT'], 'resources', 'domains', 'whois.json'];
        $whois = file_get_contents('https://manage.synergywholesale.com/home/whmcs-whois-json');
        $whoisBackup = file_get_contents($jsonPath);

        if ($whois == $whoisBackup) {
            $configarray['whoisUpdate']['Description'] = $configarray['whoisUpdate']['Description'] . "<br><b>NOTICE:</b> WHOIS.json file is already up to date.";
            return $configarray;            
        }

        //Testing to see if retrieved data is valid
        @json_decode($whois);
        if (json_last_error() !== JSON_ERROR_NONE) {
            $configarray['whoisUpdate']['Description'] = $configarray['whoisUpdate']['Description'] . "<br><b>NOTICE:</b> WHOIS.json update unsuccessful. Unable to pull file.";
            return $configarray;
        }

        //Check if file path is valid
        $filePath = realpath(implode(DIRECTORY_SEPARATOR, $jsonPath));
        if (!$filePath) {
            $configarray['whoisUpdate']['Description'] = $configarray['whoisUpdate']['Description'] . "<br><b>NOTICE:</b> WHOIS.json update unsuccessful. File path invalid. The file at " . $filePath . " does not exist.";
            return $configarray;          
        }

        if (!file_put_contents($filePath, $whois)) {
            $configarray['whoisUpdate']['Description'] = $configarray['whoisUpdate']['Description'] . "<br><b>NOTICE:</b> WHOIS.json update unsuccessful. Unable to update WHOIS.json file.";

            //Revert any changes made to backup file
            file_put_contents($filePath, $whoisBackup);

            return $configarray;  
        } else {
            $configarray['whoisUpdate']['Description'] = $configarray['whoisUpdate']['Description'] . "<br><b>NOTICE:</b> WHOIS.json successfully updated.";
        }

        try {
            // Finally disable the setting for the customer automatically so
            // we don't have them needing to turn if off manually
            Capsule::table('tblregistrars')->where(['registrar' => 'synergywholesaledomains', 'setting' => 'whoisUpdate'])->update(['value' => 'off']);
        } catch (\Exception $e) {
            // Update the description so that the client knows something oddball happened
            $configarray['whoisUpdate']['Description'] = $configarray['whoisUpdate']['Description'] . "<br><b>NOTICE:</b> WHOIS.json successfully updated however we were unable to disable this option automatically for you. Please untick the option yourself manually and click 'Save Changes' again.";
            return $configarray;
        }

    }

    return $configarray;
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_GetNameservers($params)
{

    $request = [];

    // Create the domain name from the sld and tld
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // Ask the API
    $passArr = synergywholesaledomains_APICall("getns", $request, $params);

    // Return results
    return $passArr;
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_SaveNameservers($params)
{

    $request['domainName']    = $params["sld"] . '.' . $params["tld"];
    $request['dnsConfigType'] = 1;
    if ($params["ns1"]) {
        $request['nameServers'][0] = $params["ns1"];
        $nsArr[]                   = $params["ns1"];
    }

    if ($params["ns2"]) {
        $request['nameServers'][1] = $params["ns2"];
        $nsArr[]                   = $params["ns2"];
    }

    if ($params["ns3"]) {
        $request['nameServers'][2] = $params["ns3"];
        $nsArr[]                   = $params["ns3"];
    }

    if ($params["ns4"]) {
        $request['nameServers'][3] = $params["ns4"];
        $nsArr[]                   = $params["ns4"];
    }

    if ($params["ns5"]) {
        $request['nameServers'][4] = $params["ns5"];
        $nsArr[]                   = $params["ns5"];
    }

    $ret = synergywholesaledomains_APICall("savens", $request, $params);
    if (isset($ret['error'])) {
        return $ret;
    }
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_GetRegistrarLock($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    if (!preg_match("/\.au/i", $params["tld"])) {
        // Ask the API
        $values = synergywholesaledomains_APICall("getlockstatus", $request, $params);
        // Return the results
        return $values['lockstatus'];
    }
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_SaveRegistrarLock($params)
{

    // First let's see if we are an .au domain name
    // .AU domain names cannot be locked
    if (preg_match("/\.au/i", $params["tld"])) {
        $values["error"] = "Unable to lock/unlock .AU domain names";
        return $values;
    }

    // Build the request array
    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // Ask API
    $ret = synergywholesaledomains_APICall("getlockstatus", $request, $params);

    // See what the current status is
    switch ($ret['lockstatus']) {
        case 'locked':
            // We are locked - so let's unlock then
            $res = synergywholesaledomains_APICall("unlockdomain", $request, $params);
            if (isset($res['error'])) {
                // Failed to update with API
                return $res;
            }
            break;
        case 'unlocked':
            // Domain is unlocked - let's lock it
            $res = synergywholesaledomains_APICall("lockdomain", $request, $params);
            if (isset($res['error'])) {
                // Failed to update with API
                return $res;
            }
            break;
        default:
            // We have a status we did not expect to receive
            // Return an error instead
            $values["error"] = "Unknown Domain Status, Contact Technical Support";
            return $values;
            break;
    }
}

/**
 * @param $params
 */
function synergywholesaledomains_ReleaseDomain($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    $request['tagName']    = $params['transfertag'];
    return synergywholesaledomains_APICall("releasedomain", $request, $params);
}

/**
 * @param $params
 */
function synergywholesaledomains_RegisterDomain($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    $request['years']      = $params["regperiod"];

    if ($params['idprotection'] == "1") {
        $request['idProtect'] = "Y";
    } else {
        $request['idProtect'] = "N";
    }

    if ($params["ns1"]) {
        $request['nameServers'][0] = $params["ns1"];
    }

    if ($params["ns2"]) {
        $request['nameServers'][1] = $params["ns2"];
    }

    if ($params["ns3"]) {
        $request['nameServers'][2] = $params["ns3"];
    }

    if ($params["ns4"]) {
        $request['nameServers'][3] = $params["ns4"];
    }

    if ($params["ns5"]) {
        $request['nameServers'][4] = $params["ns5"];
    }

    // Registrant Details
    $request['registrant_firstname']  = $params["firstname"];
    $request['registrant_lastname']   = $params["lastname"];
    $request['registrant_address'][0] = $params["address1"];
    $request['registrant_address'][1] = $params["address2"];
    if ($params["address2"]) {
        $request['registrant_address'][1] = $params["address2"];
    }

    $request['registrant_suburb'] = $params["city"];

    // Validate the country being specified
    if (!$country = synergywholesaledomains_validateCountry($params["country"])) {
        // Country doesn't match list
        return ["error" => "Registrant Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
    } else {
        // Country matches list entry
        $request['registrant_country'] = $country;
    }

    // See if country is AU
    if ($country == "AU") {
        // It is, so check to see if a valid AU State has been specified
        if (!$state = synergywholesaledomains_validateAUState($params["state"])) {
            // No - return an error
            return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
        } else {
            // Yes - store the state
            $request['registrant_state'] = $state;
        }
    } else {
        // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
        $request['registrant_state'] = $params["state"];
    }
    $request['registrant_postcode'] = $params["postcode"];

    // Is this an Australian Phone Number
    $phoneNumber = synergywholesaledomains_formatPhoneNumber($params["phonenumber"], $params["country"], $params["state"], $params['phonecc']);
    $faxNumber = $phonenumber;

    $request['registrant_phone']     = $phoneNumber;
    $request['registrant_fax']       = $faxNumber;
    $request['registrant_email']     = $params["email"];
    $request['technical_firstname']  = $params["adminfirstname"];
    $request['technical_lastname']   = $params["adminlastname"];
    $request['technical_address'][0] = $params["adminaddress1"];
    if ($params["adminaddress2"]) {
        $request['technical_address'][1] = $params["adminaddress2"];
    }
    $request['technical_suburb'] = $params["admincity"];

    // Validate the country being specified
    if (!$country = synergywholesaledomains_validateCountry($params["admincountry"])) {
        // Country doesn't match list
        return ["error" => "Technical Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
    } else {
        // Country matches list entry
        $request['technical_country'] = $country;
    }

    // See if country is AU
    if ($country == "AU") {
        // It is, so check to see if a valid AU State has been specified
        if (!$state = synergywholesaledomains_validateAUState($params["adminstate"])) {
            // No - return an error
            return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
        } else {
            // Yes - store the state
            $request['technical_state'] = $state;
        }
    } else {
        // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
        $request['technical_state'] = $params["adminstate"];
    }
    $request['technical_postcode'] = $params["adminpostcode"];

    $request['technical_phone']  = $phoneNumber;
    $request['technical_fax']    = $faxNumber;
    $request['technical_email']  = $params["adminemail"];
    $request['admin_firstname']  = $params["adminfirstname"];
    $request['admin_lastname']   = $params["adminlastname"];
    $request['admin_address'][0] = $params["adminaddress1"];
    if ($params["adminaddress2"]) {
        $request['admin_address'][1] = $params["adminaddress2"];
    }
    $request['admin_suburb'] = $params["admincity"];

    // Validate the country being specified
    if (!$country = synergywholesaledomains_validateCountry($params["admincountry"])) {
        // Country doesn't match list
        return ["error" => "Admin Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
    } else {
        // Country matches list entry
        $request['admin_country'] = $country;
    }

    // See if country is AU
    if ($country == "AU") {
        // It is, so check to see if a valid AU State has been specified
        if (!$state = synergywholesaledomains_validateAUState($params["adminstate"])) {
            // No - return an error
            return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
        } else {
            // Yes - store the state
            $request['admin_state'] = $state;
        }
    } else {
        // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
        $request['admin_state'] = $params["adminstate"];
    }

    $request['admin_postcode'] = $params["adminpostcode"];

    $request['admin_phone']        = $phoneNumber;
    $request['admin_fax']          = $faxNumber;
    $request['admin_email']        = $params["adminemail"];
    $request['billing_firstname']  = $params["adminfirstname"];
    $request['billing_lastname']   = $params["adminlastname"];
    $request['billing_address'][0] = $params["adminaddress1"];
    if ($params["adminaddress2"]) {
        $request['billing_address'][1] = $params["adminaddress2"];
    }
    $request['billing_suburb'] = $params["admincity"];

    // Validate the country being specified
    if (!$country = synergywholesaledomains_validateCountry($params["admincountry"])) {
        // Country doesn't match list
        return ["error" => "Billing Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
    } else {
        // Country matches list entry
        $request['billing_country'] = $country;
    }

    // See if country is AU
    if ($country == "AU") {
        // It is, so check to see if a valid AU State has been specified
        if (!$state = synergywholesaledomains_validateAUState($params["adminstate"])) {
            // No - return an error
            return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
        } else {
            // Yes - store the state
            $request['billing_state'] = $state;
        }
    } else {
        // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
        $request['billing_state'] = $params["adminstate"];
    }

    $request['billing_postcode'] = $params["adminpostcode"];
    $request['billing_phone'] = $phoneNumber;
    $request['billing_fax']   = $faxNumber;
    $request['billing_email'] = $params["adminemail"];

    if (end(explode(".", $params["tld"])) == "au") {

        $request['registrantName'] = $params["additionalfields"]['Registrant Name'];
        $request['registrantID']   = $params["additionalfields"]['Registrant ID'];

        if ($params["additionalfields"]['Registrant ID Type'] == "Business Registration Number") {
            $params["additionalfields"]['Registrant ID Type'] = "OTHER";
        }

        $request['registrantIDType'] = $params["additionalfields"]['Registrant ID Type'];
        $request['eligibilityType']  = $params["additionalfields"]['Eligibility Type'];

        if ($params["additionalfields"]['Eligibility ID Type'] == "Australian Company Number (ACN)") {
            $request['eligibilityIDType'] = "ACN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "ACT Business Number") {
            $request['eligibilityIDType'] = "ACT BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "NSW Business Number") {
            $request['eligibilityIDType'] = "NSW BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "NT Business Number") {
            $request['eligibilityIDType'] = "NT BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "QLD Business Number") {
            $request['eligibilityIDType'] = "QLD BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "SA Business Number") {
            $request['eligibilityIDType'] = "SA BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "TAS Business Number") {
            $request['eligibilityIDType'] = "TAS BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "VIC Business Number") {
            $request['eligibilityIDType'] = "VIC BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "WA Business Number") {
            $request['eligibilityIDType'] = "WA BN";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "Trademark (TM)") {
            $request['eligibilityIDType'] = "TM";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "Other - Used to record an Incorporated Association number") {
            $request['eligibilityIDType'] = "OTHER";
        } elseif ($params["additionalfields"]['Eligibility ID Type'] == "Australian Business Number (ABN)") {
            $request['eligibilityIDType'] = "ABN";
        }
        $request['eligibilityID']   = $params["additionalfields"]['Eligibility ID'];
        $request['eligibilityName'] = $params["additionalfields"]['Eligibility Name'];
    }

    // Is the domain .uk
    if (end(explode(".", $params["tld"])) == "uk") {
        $request['tradingName'] = $params["additionalfields"]['Registrant Name'];
        $request['number']      = $params["additionalfields"]['Registrant ID'];
        $request['type']        = $params["additionalfields"]['Registrant ID Type'];
        $request['optout']      = $params["additionalfields"]['WHOIS Opt-out'];
    }

    // Is the domain .us
    if (end(explode(".", $params["tld"])) == "us") {

        // Nexus Category - Sans the Country Code since we will use the registrant country code
        // to append to the NexusCategory if required
        $request['nexusCategory'] = $params["additionalfields"]['Nexus Category'];

        switch ($params["additionalfields"]['Application Purpose']) {
            case "Business use for profit":
                $request['appPurpose'] = 'P1';
                break;

            case "Non-profit business":
                $request['appPurpose'] = 'P2';
                break;

            case "Club":
                $request['appPurpose'] = 'P2';
                break;

            case "Association":
                $request['appPurpose'] = 'P2';
                break;

            case "Religious Organization":
                $request['appPurpose'] = 'P2';
                break;

            case "Personal Use":
                $request['appPurpose'] = 'P3';
                break;

            case "Educational purposes":
                $request['appPurpose'] = 'P4';
                break;

            case "Government purposes":
                $request['appPurpose'] = 'P5';
                break;

            default:
                $request['appPurpose'] = '';
                break;
        }
    }

    return synergywholesaledomains_APICall("register", $request, $params);
}

/**
 * @param $params
 */
function synergywholesaledomains_TransferDomain($params)
{

    $request = [];

    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    if (end(explode(".", $params["tld"])) != "uk") {
        $request['authInfo']     = $params["transfersecret"];
        $request['firstname']    = $params["firstname"];
        $request['lastname']     = $params["lastname"];
        $request['organisation'] = $params["companyname"];
        $request['address'][0]   = $params["address1"];
        if ($params["address2"]) {
            $request['address'][1] = $params["address2"];
        }
        $request['suburb'] = $params["city"];

        // Validate the country being specified
        if (!$country = synergywholesaledomains_validateCountry($params["country"])) {
            // Country doesn't match list
            return ["error" => "Registrant Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
        } else {
            // Country matches list entry
            $request['country'] = $country;
        }

        // See if country is AU
        if ($country == "AU") {
            // It is, so check to see if a valid AU State has been specified
            if (!$state = synergywholesaledomains_validateAUState($params["state"])) {
                // No - return an error
                return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
            } else {
                // Yes - store the state
                $request['state'] = $state;
            }
        } else {
            // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
            $request['state'] = $params["state"];
        }

        $request['postcode'] = $params["postcode"];
        $phoneNumber         = synergywholesaledomains_formatPhoneNumber($params["phonenumber"], $params["country"], $params['state']);
        $faxNumber           = synergywholesaledomains_formatPhoneNumber($params["phonenumber"], $params["country"], $params['state']);
        $request['phone']    = $phoneNumber;
        $request['fax']      = $faxNumber;
        $request['email']    = $params["email"];

        $canRenew = synergywholesaledomains_APICall("domainRenewRequired", [
        	'domainName' => $request['domainName'],
        	'authInfo' => $request['authInfo']
    	], $params);

        // See if the doRenewal value has been set to yes
        if ($params['doRenewal'] == "on" && $canRenew['status'] === 'OK_RENEW') {
            // It has, so we need to specify doRenewal
            $request['doRenewal'] = 1;
        } else {
            // It has not, so we need to set doRenewal to off
            $request['doRenewal'] = 0;
        }

        if ($params['idprotection'] == 1) {
            $request['idProtect'] = "Y";
        } else {
            $request['idProtect'] = "N";
        }
        return synergywholesaledomains_APICall("transfer", $request, $params);
    } else {
        return ["error" => "Domain name transfers for .UK domain names are completely different to other extensions. The domain name must first exist in the wholesale system before it is assigned to our tag. Once the domain has been submitted you need to request the existing registrar assign it to our tag mentioned below. Our system will automatically detect the inbound transfer to us and complete the process and set the domain live in your account. UK Tag: SYNERGY-AU"];
    }
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_IDProtectToggle($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    if ($params["protectenable"] == 1) {
        $ret = synergywholesaledomains_APICall("idprotect", $request, $params);
        if (isset($ret['error'])) {
            return $ret;
        }
    } else {
        $ret = synergywholesaledomains_APICall("idunprotect", $request, $params);
        if (isset($ret['error'])) {
            return $ret;
        }
    }
}

/**
 * @param $params
 */
function synergywholesaledomains_RenewDomain($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    $request['years']      = $params["regperiod"];
    return synergywholesaledomains_APICall("renew", $request, $params);
}

/**
 * @param $params
 */
function synergywholesaledomains_GetEmailForwarding($params)
{

    // Synergy Wholesale Module does not use this function
    return ['vars' => ['registrarModule' => $params['registrar']]];
}

/**
 * @param $params
 */
function synergywholesaledomains_SaveEmailForwarding($params)
{

    // Synergy Wholesale Module does not use this function
    return [];
}

/**
 * @param $params
 */
function synergywholesaledomains_GetDNS($params)
{

    // Synergy Wholesale Module does not use this function
    return ['vars' => ['registrarModule' => $params['registrar']]];
}

/**
 * @param  $record
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_ADDPermURLForward($record, $params)
{
    $request                = [];
    $request['domainName']  = $params["sld"] . '.' . $params["tld"];
    $request['hostName']    = $record['hostname'];
    $request['destination'] = $record['address'];
    $request['type']        = "P";
    $ret                    = synergywholesaledomains_APICall("addURLForward", $request, $params);
    return $ret;

}

/**
 * @param  $record
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_ADDCloakURLForward($record, $params)
{
    $request                = [];
    $request['domainName']  = $params["sld"] . '.' . $params["tld"];
    $request['hostName']    = $record['hostname'];
    $request['destination'] = $record['address'];
    $request['type']        = "C";
    $ret                    = synergywholesaledomains_APICall("addURLForward", $request, $params);
    return $ret;

}

/**
 * @param  $record
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_DELURLForward($record, $params)
{
    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    $request['recordID']   = $record['recordid'];
    $ret                   = synergywholesaledomains_APICall("deleteURLForward", $request, $params);
    return $ret;

}

/**
 * @param  $record
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_AddDNSRec($record, $params)
{

    $request = [];

    $request['domainName']    = $params["sld"] . '.' . $params["tld"];
    $request['recordName']    = $record['hostname'];
    $request['recordType']    = $record['type'];
    $request['recordContent'] = $record['address'];

    // See if the TTL has been specified
    if (isset($record['ttl'])) {
        $request['recordTTL'] = $record['ttl'];
    } else {
        // Set default ttl of 14400
        $request['recordTTL'] = 14400;
    }

    // See if priority has been specified
    if (isset($record['priority'])) {
        $request['recordPrio'] = $record['priority'];
    }

    if ($request['recordType'] == 'NS' && $request['recordName'] == $request['domainName']) {
        return [
            'error' => "Cannot add or remove NS records from root domain."
        ];
    }

    $ret = synergywholesaledomains_APICall("addDNSRecord", $request, $params);
    return $ret;
}

/**
 * @param  $record
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_DelDNSRec($record, $params)
{

    $request = [];

    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    $request['recordID']   = $record['recordid'];
    $ret                   = synergywholesaledomains_APICall("deleteDNSRecord", $request, $params);
    return $ret;
}

/*
 *    This function will save any dns records to the database
 */
/**
 * @param $params
 */
function synergywholesaledomains_SaveDNS($params)
{

    // Synergy Wholesale Module does not use this function
    return [];
}

/**
 * @param $params
 */
function synergywholesaledomains_Sync($params)
{
    // Run the sync command on the domain specified
    $response = synergywholesaledomains_APICall('sync', [
        'domainName' => $params['sld'] . '.' . $params['tld']
    ], $params);

    // See if we got an error
    if (isset($response['error'])) {
        return $response;
    }
   
    if (isset($response['transfer_status'])) {
        return synergywholesaledomains_TransferSync($params);
    }

    $returnData = [];
    if (preg_match('/\.au$/', $params['tld'])) {
        $appMap = [
            'auRegistrantIDType' => 'Registrant ID Type',
            'auRegistrantID' => 'Registrant ID',
            'auRegistrantName' => 'Registrant Name',
            'auEligibilityName' => 'Eligibility Name',
            'auEligibilityID' => 'Eligibility ID',
            'auEligibilityType' => 'Eligibility Type',
            'auEligibilityIDType' => 'Eligibility ID Type',
            'auPolicyID' => 'Eligibility Reason',
        ];
        try {
            foreach ($appMap as $apiName => $whmcsName) {
                if (empty($apiResult[$apiName])) {
                    continue;
                }
                if ($apiName == 'auPolicyID') {
                    switch ($apiResult[$apiName]) {
                        case 1:
                            $apiResult[$apiName] = 'Domain name is an Exact Match Abbreviation or Acronym of your Entity or Trading Name.';
                            break;
                        case 2:
                            $apiResult[$apiName] = 'Close and substantial connection between the domain name and the operations of your Entity.';
                            break;
                    }
                }
                Capsule::table('tbldomainsadditionalfields')
                    ->where('domainid', $params['domainid'])
                    ->where('name', $whmcsName)
                    ->update([
                        'value' => $apiResult[$apiName]
                    ])
                ;
            }
        } catch (\Exception $e) {
            logModuleCall('synergywholesaledomains', 'sync_process', 'Update DB', $e->getMessage());
        }
    }

    try {
        $selectInfo = Capsule::table('tbldomains')
            ->select('expirydate', 'additionalnotes', 'status')
            ->where('id', $params['domainid'])
            ->first();
        // If the domain used to exist in this whmcs installation it's safe to say if we get these errors then
        // it has been transferred away to another reseller
        if ($response['errorMessage'] == 'Domain Info Failed - Unable to retrieve domain id') {
            // If now is after the domains expiry date mark it as cancelled
            if (time() >= strtotime($selectInfo->expirydate)) {
                $note = 'Domain has been marked as cancelled due to not being in your account and, the current date is past the expiry date';
                $returnData['cancelled'] = true;
            } else {
                $note = 'Domain has been marked as transferred away due to not being in your account';
                $returnData['transferredAway'] = true;
            }
        } elseif (!isset($response['domain_status'])) {
            return [
                'active' => $selectInfo->status == 'Active',
                'expired' => $selectInfo->status != 'Active'
            ];
        } else {
            switch (strtolower($response['domain_status'])) {
                case 'ok':
                case 'clienttransferprohibited':
                case 'inactive':
                    $returnData = [
                        'active' => true,
                        'expirydate' => substr($response['domain_expiry'], 0, 10)
                    ];
                    break;
                case 'expired':
                case 'clienthold':
                case 'redemption':

                    $returnData = [
                        'expired' => true,
                        'expirydate' => substr($response['domain_expiry'], 0, 10)
                    ];
                    break;
                case 'outbound':
                case 'outbound_emailed':
                case 'transferaway':
                case 'outbound_approved':
                    $note = 'Domain is transferring out of your reseller account';
                    $returnData = [
                        'active' => true
                    ];
                    break;
                case 'deleted':
                case 'dropped':
                case 'policydelete':
                    $note = "Domain has been marked as cancelled due to being deleted";
                    $returnData = [ // Double check this is actually an okay thing.
                        'cancelled' => true
                    ];
                    break;
                case 'transferredaway':
                case 'domain does not exist':
                    $note = 'Domain has transferred out of your partner account';
                    $returnData = [
                        'transferredAway' => true
                    ];
                    break;
                default:
                    $returnData = [
                        'active' => true
                    ];
                    if (isset($response['domain_expiry'])) {
                        $returnData['expirydate'] = substr($response['domain_expiry'], 0, 10);
                    }
                    break;
            }
        }

        if (isset($note)) {
            Capsule::table('tbldomains')
                ->where('id', $params['domainid'])
                ->update([
                    'additionalnotes' => $selectInfo->additionalnotes . "\r\n" . date('d/m/Y') . ' - Sync Cron - ' . $note
                ]);
        }
    } catch (\Exception $e) {
        logModuleCall('synergywholesaledomains', 'sync_process', 'Update DB', $e->getMessage());
    }

    return $returnData;
}

function synergywholesaledomains_TransferSync($params)
{
    $response = synergywholesaledomains_APICall("sync", [
        'domainName' => $params['sld'] . '.' . $params['tld']
    ], $params);
    
    if (isset($response['error'])) {
        return ['error' => $response['error']];
    }

    if (!isset($response['domain_status'])) {
        return ['completed' => false];
    }

    switch (strtolower($response['domain_status'])) {
        case 'ok':
        case 'clienttransferprohibited':
        case 'clienthold':
        case 'expired':
        case 'inactive':
            return [
                'completed' => true,
                'expirydate' => substr($response['domain_expiry'], 0, 10),
                'failed' => false
            ];
            break;
        case 'transfer_rejected':
        case 'transfer_cancelled':
        case 'transfer_cancelled':
        case 'transfer_rejected_registry':
        case 'transfer_timeout':
            return [
                'completed' => false,
                'failed' => true, 
                'reason' => 'Transfer was either rejected, cancelled or timed out'
            ];
            break;
        default:
            return [
                'completed' => false
            ];
    }
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_SaveContactDetails($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // Build registrant data array
    $request['registrant_firstname'] = $params["contactdetails"]["Registrant"]["First Name"];
    $request['registrant_lastname']  = $params["contactdetails"]["Registrant"]["Last Name"];
    $request['registrant_address']   = [$params["contactdetails"]["Registrant"]["Address 1"], $params["contactdetails"]["Registrant"]["Address 2"]];
    $request['registrant_email']     = $params["contactdetails"]["Registrant"]["Email"];
    $request['registrant_suburb']    = $params["contactdetails"]["Registrant"]["City"];
    $request['registrant_postcode']  = $params["contactdetails"]["Registrant"]["Postcode"];

    // Validate the country being specified
    if (!$country = synergywholesaledomains_validateCountry($params["contactdetails"]["Registrant"]["Country"])) {
        // Country doesn't match list
        return ["error" => "Registrant Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
    } else {
        // Country matches list entry
        $request['registrant_country'] = $country;
    }

    // See if country is AU
    if ($country == "AU") {
        // It is, so check to see if a valid AU State has been specified
        if (!$state = synergywholesaledomains_validateAUState($params["contactdetails"]["Registrant"]["State"])) {
            // No - return an error
            return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
        } else {
            // Yes - store the state
            $request['registrant_state'] = $state;
        }
    } else {
        // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
        $request['registrant_state'] = $params["contactdetails"]["Registrant"]["State"];
    }

    $phone = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Registrant"]["Phone"], $country, $state, $params["contactdetails"]["Registrant"]["Phone Country Code"]);
    $fax   = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Registrant"]["Fax"], $country, $state, $params["contactdetails"]["Registrant"]["Phone Country Code"]);

    $request['registrant_phone'] = $phone;
    $request['registrant_fax']   = $fax;
    
    // See if any admin contact data was passed back

    if (isset($params["contactdetails"]["Admin"])) {
        // Build admin data array
        $request['admin_firstname'] = $params["contactdetails"]["Admin"]["First Name"];
        $request['admin_lastname']  = $params["contactdetails"]["Admin"]["Last Name"];
        $request['admin_address']   = [$params["contactdetails"]["Admin"]["Address 1"], $params["contactdetails"]["Admin"]["Address 2"]];
        $request['admin_suburb']    = $params["contactdetails"]["Admin"]["City"];
        $request['admin_postcode']  = $params["contactdetails"]["Admin"]["Postcode"];
        $request['admin_email']     = $params["contactdetails"]["Admin"]["Email"];

        // Validate the country being specified
        if (!$country = synergywholesaledomains_validateCountry($params["contactdetails"]["Admin"]["Country"])) {
            // Country doesn't match list
            return ["error" => "Admin Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
        } else {
            // Country matches list entry
            $request['admin_country'] = $country;
        }

        // See if country is AU
        if ($country == "AU") {
            // It is, so check to see if a valid AU State has been specified
            if (!$state = synergywholesaledomains_validateAUState($params["contactdetails"]["Admin"]["State"])) {
                // No - return an error
                return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
            } else {
                // Yes - store the state
                $request['admin_state'] = $state;
            }
        } else {
            // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
            $request['admin_state'] = $params["contactdetails"]["Admin"]["State"];
        }

        $phone = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Admin"]["Phone"], $country, $state, $params["contactdetails"]["Admin"]["Phone Country Code"]);
        $fax   = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Admin"]["Fax"], $country, $state, $params["contactdetails"]["Admin"]["Phone Country Code"]);

        $request['admin_phone'] = $phone;
        $request['admin_fax']   = $fax;

    }

    // See if any technical details were passed back

    if (isset($params["contactdetails"]["Tech"])) {

        // Build tech data array
        $request['technical_firstname'] = $params["contactdetails"]["Tech"]["First Name"];
        $request['technical_lastname']  = $params["contactdetails"]["Tech"]["Last Name"];
        $request['technical_address']   = [$params["contactdetails"]["Tech"]["Address 1"], $params["contactdetails"]["Tech"]["Address 2"]];
        $request['technical_suburb']    = $params["contactdetails"]["Tech"]["City"];
        $request['technical_postcode']  = $params["contactdetails"]["Tech"]["Postcode"];
        $request['technical_email']     = $params["contactdetails"]["Tech"]["Email"];

        // Validate the country being specified
        if (!$country = synergywholesaledomains_validateCountry($params["contactdetails"]["Tech"]["Country"])) {
            // Country doesn't match list
            return ["error" => "Tech Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
        } else {
            // Country matches list entry
            $request['technical_country'] = $country;
        }

        // See if country is AU
        if ($country == "AU") {
            // It is, so check to see if a valid AU State has been specified
            if (!$state = synergywholesaledomains_validateAUState($params["contactdetails"]["Tech"]["State"])) {
                // No - return an error
                return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
            } else {
                // Yes - store the state
                $request['technical_state'] = $state;
            }
        } else {
            // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
            $request['technical_state'] = $params["contactdetails"]["Tech"]["State"];
        }

        $phone = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Tech"]["Phone"], $country, $state, $params["contactdetails"]["Tech"]["Phone Country Code"]);
        $fax   = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Tech"]["Fax"], $country, $state, $params["contactdetails"]["Tech"]["Phone Country Code"]);

        $request['technical_phone'] = $phone;
        $request['technical_fax']   = $fax;

    }

    // See if any billing details were passed back
    if (isset($params["contactdetails"]["Billing"])) {

        // Build billing data array
        $request['billing_firstname'] = $params["contactdetails"]["Billing"]["First Name"];
        $request['billing_lastname']  = $params["contactdetails"]["Billing"]["Last Name"];
        $request['billing_address']   = [$params["contactdetails"]["Billing"]["Address 1"], $params["contactdetails"]["Billing"]["Address 2"]];
        $request['billing_suburb']    = $params["contactdetails"]["Billing"]["City"];
        $request['billing_postcode']  = $params["contactdetails"]["Billing"]["Postcode"];
        $request['billing_email']     = $params["contactdetails"]["Billing"]["Email"];

        // Validate the country being specified
        if (!$country = synergywholesaledomains_validateCountry($params["contactdetails"]["Billing"]["Country"])) {
            // Country doesn't match list
            return ["error" => "Billing Country must be entered as 2 characters - ISO 3166 Standard. EG. AU"];
        } else {
            // Country matches list entry
            $request['billing_country'] = $country;
        }

        // See if country is AU
        if ($country == "AU") {
            // It is, so check to see if a valid AU State has been specified
            if (!$state = synergywholesaledomains_validateAUState($params["contactdetails"]["Billing"]["State"])) {
                // No - return an error
                return ["error" => "A Valid Australian State Name Must Be Supplied, EG. NSW, VIC"];
            } else {
                // Yes - store the state
                $request['billing_state'] = $state;
            }
        } else {
            // Country is not Australia, so we can just use whatever has been supplied as we can't validate it
            $request['billing_state'] = $params["contactdetails"]["Billing"]["State"];
        }

        $phone = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Billing"]["Phone"], $country, $state, $params["contactdetails"]["Billing"]["Phone Country Code"]);
        $fax = synergywholesaledomains_formatPhoneNumber($params["contactdetails"]["Billing"]["Fax"], $country, $state, $params["contactdetails"]["Billing"]["Phone Country Code"]);
        $request['billing_phone'] = $phone;
        $request['billing_fax']   = $fax;
    }

    // Execute command
    $ret = synergywholesaledomains_APICall("savecontacts", $request, $params);
    if (isset($ret['error'])) {
        // Unable to save the contacts
        return $ret;
    }

}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_GetContactDetails($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // We need to get status from API
    $idProtectStatus = synergywholesaledomains_APICall("getidprotectionstatus", $request, $params);
    // See if Domain ID Protection is enabled
    if ($idProtectStatus['idProtect'] == "Enabled") {
        // Yes - We don't allow contact updates to contacts with id protection enabled
        $values['error'] = "ID Protection Is Enabled - Unable To Update Domain Contacts - Visit the Addons page in the Manage Menu to Disable ID Protection";
        return $values;
    } else {
        // Grab the contacts
        return synergywholesaledomains_APICall("getcontacts", $request, $params);
    }
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_GetEPPCode($params)
{

    $request               = [];
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    // Execute the geteppcode function
    $eppCode = synergywholesaledomains_APICall("geteppcode", $request, $params);
    // Store the eppCode
    $values["eppcode"] = $eppCode['eppCode'];
    // Return values
    return $values;
}

/**
 * @param $params
 */
function synergywholesaledomains_domainOptions($params)
{

    // Map domainid, and domain name to friendly vars
    $domainid = $params['domainid'];
    $sld      = $params['sld'];
    $tld      = $params['tld'];

    // Do a quick query to see if dnsmanagement or emailforwarding are enabled
    $domainInfo = Capsule::table('tbldomains')->select('dnsmanagement', 'emailforwarding')->where('id', $domainid)->get();

    // Create empty arrays
    $request  = [];
    $vars     = [];
    $errorArr = [];

    // See if dnsmanagement or emailforwarding are enabled for the domain
    if ((int) $domainInfo[0]->dnsmanagement == 1) {
        $vars['dnsmanagement'] = 1;
    } else {
        $vars['dnsmanagement'] = 0;
    }
    if ((int) $domainInfo[0]->emailforwarding == 1) {
        $vars['emailforwarding'] = 1;
    } else {
        $vars['emailforwarding'] = 0;
    }

    // Map the domain name to $_REQUEST array
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];
    $vars['tld']           = $params["tld"];

    // Post handler
    if (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'save') {

        // See what opt we are performing
        if (isset($_REQUEST['opt']) && $_REQUEST['opt'] == "dnstype") {

            // Execute the Get Nameservers request
            $nameServers = synergywholesaledomains_APICall("getns", $request, $params);
            // If any errors detected, store in the errorArr
            if ($nameServers['error']) {
                $errorArr[] = 'Get Name Servers failed: ' . $nameServers['error'];
            }

            // Confirm if the dnsConfigType set is currently custom
            if ($nameServers['dnsConfigType'] == 1) {
                // It's custom at present, we need to set to the freedns system nameservers
                $request['nameServers'][0] = "ns1.nameserver.net.au";
                $request['nameServers'][1] = "ns2.nameserver.net.au";
                $request['nameServers'][2] = "ns3.nameserver.net.au";
            } else {
                // Map the name servers from the function call 'getns'
                if ($nameServers["ns1"]) {
                    $request['nameServers'][0] = $nameServers["ns1"];
                }
                if ($nameServers["ns2"]) {
                    $request['nameServers'][1] = $nameServers["ns2"];
                }
                if ($nameServers["ns3"]) {
                    $request['nameServers'][2] = $nameServers["ns3"];
                }
                if ($nameServers["ns4"]) {
                    $request['nameServers'][3] = $nameServers["ns4"];
                }
                if ($nameServers["ns5"]) {
                    $request['nameServers'][4] = $nameServers["ns5"];
                }
            }

            // Set the dnsConfigType option
            $request['dnsConfigType'] = $_REQUEST['option'];

            $ret = synergywholesaledomains_APICall("updateDnsType", $request, $params);
            // If any errors detected, store in the errorArr
            if ($ret['error']) {
                $errorArr[] = 'Update DNS Config Type failed: ' . $ret['error'];
            }

        } elseif (isset($_REQUEST['opt']) && $_REQUEST['opt'] == "xxxmembership") {
            // Build the request
            $request['membershipToken'] = $_REQUEST['xxxToken'];
            // Send the request
            $ret = synergywholesaledomains_APICall("updateXXXMembership", $request, $params);
            if ($ret['error']) {
                $errorArr[] = 'Update XXX Membership failed: ' . $ret['error'];
            } else {
                $vars['info'] = 'Update XXX Membership successful';
            }
        } elseif (isset($_REQUEST['opt']) && $_REQUEST['opt'] == "resendwhoisverif") {
            // send the request
            $ret = synergywholesaledomains_APICall("resendWhoisVerif", $request, $params);
            if ($ret['error']) {
                $errorArr[] = 'Resend Whois Verification Email failed: ' . $ret['error'];
            } else {
                $vars['info'] = 'Resend Whois Verification Email successfully resent';
            }
        }
    }

    // Call From API
    $ret = synergywholesaledomains_APICall("dnsConfigType", $request, $params);
    if ($ret['error']) {
        $errorArr[] = 'Get DNS Config Type failed: ' . $ret['error'];
    } else {
        $vars['dnsConfigType'] = $ret['dnsConfigType'];
    }

    // Call From API
    $ret = synergywholesaledomains_APICall("getICANNStatus", $request, $params);
    if ($ret['error']) {
        $errorArr[] = 'Get ICANN Status failed: ' . $ret['error'];
    } else {
        $vars['icannStatus'] = $ret['icannStatus'];
    }

    // Build the error variable should it exist
    $errString = '';
    foreach ($errorArr as $error) {
        $errString .= $error . "<br>";
    }

    // Map it to the vars
    if (strlen($errString) > 0) {
        $vars['error'] = $errString;
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'save') {
        $vars['info'] = 'Domain options have been updated successfully';
    }

    // Return specific template details to smarty
    return [
        'templatefile' => 'domainoptions',
        'breadcrumb'   => [
            'clientarea.php?action=domaindetails&domainid=' . $domainid . '&modop=custom&a=domainOptions' => 'Domain Options',
        ],
        'vars'         => $vars,
    ];
}

/**
 * @param $params
 */
function synergywholesaledomains_manageDNSSEC($params)
{

    // Map domainid, and domain name to friendly vars
    $domainid = $params['domainid'];
    $sld      = $params['sld'];
    $tld      = $params['tld'];

    // Create empty arrays
    $request  = [];
    $vars     = [];
    $errorArr = [];

    // Map the domain name to $_REQUEST array
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // Post handler
    if (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'save') {
        // This is adding a new dnssec record
        // build the request
        $request['algorithm']  = $_REQUEST['algorithm'];
        $request['digestType'] = $_REQUEST['digestType'];
        $request['digest']     = $_REQUEST['digest'];
        $request['keyTag']     = $_REQUEST['keyTag'];
        // Call the add function
        $ret = synergywholesaledomains_APICall("addDNSSECEntry", $request, $params);
        if ($ret['error']) {
            $errorArr[] = $ret['error'];
        } else {
            // We need to sync up the dnssec records now, since we have to get a uuid from the api
            $ret = synergywholesaledomains_APICall("getDNSSECEntries", $request, $params);
        }
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'delete') {
        // This is a delete request
        // build the request
        $request['UUID'] = $_REQUEST['uuid'];
        // Call the delete function
        $ret = synergywholesaledomains_APICall("deleteDNSSECEntry", $request, $params);
        if ($ret['error']) {
            $errorArr[] = $ret['error'];
        }
    }

    // Get a current list of any dnssec records
    $ret = synergywholesaledomains_APICall("getDNSSECEntries", $request, $params);
    if ($ret['error']) {
        $errorArr[] = $ret['error'];
    } else {
        $vars['records'] = $ret;
    }

    // Build the error variable should it exist
    $errString = '';
    foreach ($errorArr as $error) {
        $errString .= $error . "<br>";
    }

    // Map it to the vars
    if (strlen($errString) > 0) {
        $vars['error'] = $errString;
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'save') {
        $vars['info'] = 'DNSSEC Record added successfully';
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'delete') {
        $vars['info'] = 'DNSSEC Record deleted successfully';
    }

    // Return specific template details to smarty
    return [
        'templatefile' => 'domaindnssec',
        'breadcrumb'   => [
            'clientarea.php?action=domaindetails&domainid=' . $domainid . '&modop=custom&a=manageDNSSEC' => 'Manage DNSSEC Records',
        ],
        'vars'         => $vars,
    ];
}

/**
 * @param $params
 */
function synergywholesaledomains_manageChildHosts($params)
{

    // Map domainid, and domain name to friendly vars
    $domainid = $params['domainid'];
    $sld      = $params['sld'];
    $tld      = $params['tld'];

    // Create empty arrays
    $request  = [];
    $vars     = [];
    $errorArr = [];

    // Map the domain name to an easy to use var
    $domainName = $params["sld"] . '.' . $params["tld"];

    // Map the domain name to $_REQUEST array
    $request['domainName'] = $domainName;

    // Post handler
    if (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Manage Host') {
        // This is Managing a host record
        // We simply need to let smarty know this should render the manage host screen
        $vars['manageHost'] = 1;
        $vars['hostname']   = $_REQUEST['ipHost'];
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Delete Host') {
        // This is a delete request

        if (preg_match("/(.*)\.$domainName/", $_REQUEST['ipHost'], $regs)) {
            $ipHost = $regs[1];
            // build the request
            $request['host'] = $ipHost;
            // Call the delete function
            $ret = synergywholesaledomains_APICall("delhost", $request, $params);
            if ($ret['error']) {
                $errorArr[] = $ret['error'];
            }
        } else {
            $errorArr[] = "Unable to determine the hostname of the child host record";
        }

    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Save Host') {
        // This is creating a new child host record
        // build the request
        $request['host']      = $_REQUEST['newHostName'];
        $request['ipAddress'] = [$_REQUEST['newIPAddress']];
        // Call the delete function
        $ret = synergywholesaledomains_APICall("addhost", $request, $params);
        if ($ret['error']) {
            $errorArr[] = $ret['error'];
        }

    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Delete Host IP') {

        // This is a delete request for host ip

        if (preg_match("/(.*)\.$domainName/", $_REQUEST['ipHost'], $regs)) {
            $ipHost = $regs[1];
            // build the request
            $request['host']      = $ipHost;
            $request['ipAddress'] = [$_REQUEST['ipRecord']];
            // Call the delete function
            $ret = synergywholesaledomains_APICall("delhostip", $request, $params);
            if ($ret['error']) {
                $errorArr[] = $ret['error'];
            }
        } else {
            $errorArr[]         = "Unable to determine the hostname of the child host record";
            $vars['manageHost'] = 1;
            $vars['hostname']   = $_REQUEST['ipHost'];
        }

    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Save Host IP') {

        // This is a request to save new ip to the system
        if (preg_match("/(.*)\.$domainName/", $_REQUEST['ipHost'], $regs)) {
            $ipHost = $regs[1];
            // build the request
            $request['host']      = $ipHost;
            $request['ipAddress'] = [$_REQUEST['newIPAddress']];
            // Call the add function
            $ret = synergywholesaledomains_APICall("addhostip", $request, $params);
            if ($ret['error']) {
                $errorArr[] = $ret['error'];
            }
        } else {
            $errorArr[]         = "Unable to determine the hostname of the child host record";
            $vars['manageHost'] = 1;
            $vars['hostname']   = $_REQUEST['ipHost'];
        }
    }

    // Check API for child host records
    $ret = synergywholesaledomains_APICall("getAllChildHosts", $request, $params);
    if ($ret['error']) {
        // See if this is a No Host Records Present Error?
        if (!preg_match("/No Host Records Present/i", $ret['error'])) {
            // This is an actual error
            $errorArr[] = $ret['error'];
        } else {
            $vars['info'] = "No host records have been found for this domain name";
        }
    } else {
        $vars['records'] = $ret;
    }

    // Build the error variable should it exist
    $errString = '';
    foreach ($errorArr as $error) {
        $errString .= $error . "<br>";
    }

    // Map it to the vars
    if (strlen($errString) > 0) {
        $vars['error'] = $errString;
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Delete Host') {
        $vars['info'] = 'Child Host has been successfully deleted';
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Delete Host IP') {
        $vars['info']       = 'IP removed successfully from child host record';
        $vars['manageHost'] = 1;
        $vars['hostname']   = $_REQUEST['ipHost'];
    } elseif (isset($_REQUEST['sub']) && $_REQUEST['sub'] == 'Save Host IP') {
        $vars['info']       = 'IP added successfully to child host record';
        $vars['manageHost'] = 1;
        $vars['hostname']   = $_REQUEST['ipHost'];
    }

    // Return specific template details to smarty
    return [
        'templatefile' => 'domainchildhosts',
        'breadcrumb'   => [
            'clientarea.php?action=domaindetails&domainid=' . $domainid . '&modop=custom&a=manageChildHosts' => 'Manage Child Host Records',
        ],
        'vars'         => $vars,
    ];
}

/**
 * @param $params
 */
function synergywholesaledomains_manageDNSURLForwarding($params)
{

    // Map domainid, and domain name to friendly vars
    $domainid = $params['domainid'];
    $sld      = $params['sld'];
    $tld      = $params['tld'];

    // Map the FreeDNS Hosting Nameservers
    $freeDNSNS = ['ns1.nameserver.net.au', 'ns2.nameserver.net.au', 'ns3.nameserver.net.au'];

    // Map the domain name
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // See if we have been requested to load records
    if (isset($_REQUEST['op']) && $_REQUEST['op'] == "getRecords") {
        $oldexistingrecords = synergywholesaledomains_custom_GetDNS($params);
        if (is_array($oldexistingrecords) && isset($oldexistingrecords['error'])) {
            // Check to see if any errors
            if (preg_match('/DNS Hosting Is Not Enabled For This Domain/i', $oldexistingrecords["error"])) {
                // Just means the system is not enabled for it
                $existingrecords         = [];
                $existingrecords["info"] = "NOTE: It appears that DNS Hosting is not enabled for this domain name. Any records you add will automatically update the nameservers set on the domain name which may result in an undesired outcome which could result in your website and email being taken offline.";
            } else {
                // Standard error
                $existingrecords          = [];
                $existingrecords["error"] = $oldexistingrecords['error'];
            }
        } elseif (is_array($oldexistingrecords)) {
            // We have records (apparently)
            // Confirm this
            if (count($oldexistingrecords) > 0) {
                $existingrecords = $oldexistingrecords;
            } else {
                $existingrecords         = [];
                $existingrecords["info"] = "No records exist for this domain name";
            }
        } else {
            $existingrecords         = [];
            $existingrecords["info"] = "No records exist for this domain name";
        }

        // If existing records is an actual array rather just reporting info.
        // Add the domain name to the DNS records.
        if (is_array($existingrecords) &&
            $existingrecords["info"] == null &&
            $existingrecords["error"] == null) {
            foreach ($existingrecords as &$record) {
                $record["domain"] = $request['domainName'];
            }
        }

        // Print results in JSON and exit the code
        // This part of the code is called by AJAX
        // so we dont want to render the tempaltes, etc
        print json_encode($existingrecords);
        exit();

    } elseif (isset($_REQUEST['op']) && $_REQUEST['op'] == "deleteRecord") {
        if (isset($_REQUEST['recordid']) && strlen($_REQUEST['recordid']) > 0) {
            $dnsrecord['recordid'] = $_REQUEST['recordid'];
        }

        // If it's a URL Forward, delete it
        if ($_REQUEST['type'] == "URL" || $_REQUEST['type'] == "FRAME") {
            $ret = synergywholesaledomains_DELURLForward($dnsrecord, $params);
            if ($ret["error"]) {
                $resp["error"] = "Error Deleting URL forward from database: " . $ret["error"];
            } else {
                $resp["info"] = "URL forwarder has been deleted";
            }
        // If it's an NS record on the root domain, don't delete it
        } elseif ($_REQUEST['type'] == "NS" && $_REQUEST["hostname"] == $request['domainName']) {
            $resp["error"] = "Error Deleting DNS record from database: Cannot add or remove NS records from root domain.";
        } else {
            // Anything else, remove it as well
            $ret = synergywholesaledomains_DelDNSRec($dnsrecord, $params);
            if ($ret["error"]) {
                $resp["error"] = "Error Deleting DNS record from database: " . $ret["error"];
            } else {
                $resp["info"] = "DNS record has been deleted";
            }
        }

        // Print results in JSON and exit the code
        // This part of the code is called by AJAX
        // so we dont want to render the templates, etc
        print json_encode($resp);
        exit();

    } elseif (isset($_REQUEST['op']) && $_REQUEST['op'] == "addRecord") {

        // We are adding a record, should we set the freedns name servers

        // First we need to confirm that we have the right servers and configtype
        $getns = synergywholesaledomains_APICall("getns", $request, $params);
        if (isset($getns['error'])) {
            $resp["error"] = "Unable to get the currently configured name servers";
            print json_encode($resp);
            exit();
        }

        // Loop through the returned results
        $correctNSCount = 0;
        foreach ($getns as $key => $value) {
            if (preg_match('/ns[1-3]/', $key)) {
                if (in_array($value, $freeDNSNS)) {
                    $correctNSCount++;
                }
            }
            if ($key == "dnsConfigType") {
                $dnsConfigType = $value;
            }
        }

        // See if we match all conditions required
        if ($correctNSCount != 3 || ($dnsConfigType != 2 && $dnsConfigType != 4)) {
            // We need to update the nameservers
            $newRequest['domainName']     = $params["sld"] . '.' . $params["tld"];
            $newRequest['dnsConfigType']  = 2;
            $newRequest['nameServers'][0] = "ns1.nameserver.net.au";
            $newRequest['nameServers'][1] = "ns2.nameserver.net.au";
            $newRequest['nameServers'][2] = "ns3.nameserver.net.au";
            $savens                       = synergywholesaledomains_APICall("savens", $newRequest, $params);
            if (isset($savens['error'])) {
                $resp["error"] = "Unable to set the DNS Hosting Nameservers";
                print json_encode($resp);
                exit();
            }
        }

        if (isset($_REQUEST['hostname']) && strlen($_REQUEST['hostname']) > 0) {
            $dnsrecord['hostname'] = preg_replace("(^https?://)", "", strtolower($_REQUEST['hostname']));
            if (substr($dnsrecord['hostname'], -1) == ".") {
                $dnsrecord['hostname'] = substr($dnsrecord['hostname'], 0, -1);
            }
            $domain = $params["sld"] . '.' . $params["tld"];
            if (!preg_match('/' . $domain . '$/i',$dnsrecord['hostname'])) {
                $dnsrecord['hostname'] = $dnsrecord['hostname'] . "." . $domain;
            }
        } else {
            // This may just be an empty record, which means we just use the domain name
            $dnsrecord['hostname'] = $params["sld"] . '.' . $params["tld"];
        }
        if (isset($_REQUEST['type']) && strlen($_REQUEST['type']) > 0) {
            $dnsrecord['type'] = $_REQUEST['type'];
        }
        if (isset($_REQUEST['address']) && strlen($_REQUEST['address']) > 0) {
            $dnsrecord['address'] = $_REQUEST['address'];
        }
        if (isset($_REQUEST['priority']) && strlen($_REQUEST['priority']) > 0) {
            $dnsrecord['priority'] = $_REQUEST['priority'];
        }
        if (isset($_REQUEST['ttl']) && strlen($_REQUEST['ttl']) > 0) {
            $dnsrecord['ttl'] = $_REQUEST['ttl'];
        }

        if ($dnsrecord['type'] == "URL") {
            $ret = synergywholesaledomains_ADDPermURLForward($dnsrecord, $params);
            if ($ret["error"]) {
                $resp["error"] = "Error Adding URL Permanent forward to database: " . $ret["error"];
            } else {
                $resp = array_merge(['info' => 'Permanent URL forward has been created'], $ret);
            }
        } elseif ($dnsrecord['type'] == "FRAME") {
            $ret = synergywholesaledomains_ADDCloakURLForward($dnsrecord, $params);
            if ($ret["error"]) {
                $resp["error"] = "Error Adding URL Cloak forward to database: " . $ret["error"];
            } else {
                $resp = array_merge(['info' => 'URL Cloaking forward has been created'], $ret);
            }
        } elseif ($dnsrecord['type'] == "NS" && $dnsrecord['hostname'] == $request['domainName']) {
            $resp["error"] = "Error adding DNS record to database: Cannot add or remove NS from root domain.";
        } else {
            $ret = synergywholesaledomains_AddDNSRec($dnsrecord, $params);
            if ($ret["error"]) {
                $resp["error"] = "Error adding DNS record to database: " . $ret["error"];
            } else {
                // We have data, update the recordName field to not have the domain in it
                $ret['recordName'] = preg_replace('/\.' . $ret['domainName'] . '.?\s*$/m', '', $ret['recordName']);
                $resp              = array_merge(['info' => "DNS record has been created"], $ret);
            }
        }

        // Print results in JSON and exit the code
        // This part of the code is called by AJAX
        // so we dont want to render the tempaltes, etc
        print json_encode($resp);
        exit();
    }

    // Return specific template details to smarty
    return [
        'templatefile' => 'domaindnsurlforwarding',
        'breadcrumb'   => [
            'clientarea.php?action=domaindetails&domainid=' . $domainid . '&modop=custom&a=manageDNSURLForwarding' => 'DNS Hosting / URL Forwarding',
        ],
        'vars'         => $vars,
    ];
}

/**
 * @param $params
 */
function synergywholesaledomains_manageEmailForwarding($params)
{

    // Map domainid, and domain name to friendly vars
    $domainid = $params['domainid'];
    $sld      = $params['sld'];
    $tld      = $params['tld'];

    // Map the FreeDNS Hosting Nameservers
    $freeDNSNS = ['ns1.nameserver.net.au', 'ns2.nameserver.net.au', 'ns3.nameserver.net.au'];

    // Grab the existing records
    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // See if we have been requested to load records
    if (isset($_REQUEST['op']) && $_REQUEST['op'] == "getRecords") {
        // Get the existing records
        $oldexistingrecords = synergywholesaledomains_APICall("listMailForwards", $request, $params);
        if (is_array($oldexistingrecords) && isset($oldexistingrecords['error'])) {
            // Check to see if any errors
            if (preg_match('/Email Forwarding Is Not Enabled For This Domain/i', $oldexistingrecords["error"])) {
                // Just means the system is not enabled for it
                $existingrecords         = [];
                $existingrecords["info"] = "NOTE: It appears that DNS Hosting is not enabled for this domain name. Any records you add will automatically update the nameservers set on the domain name which may result in an undesired outcome which could result in your website and email being taken offline.";
            } else {
                // Standard error
                $existingrecords          = [];
                $existingrecords["error"] = $oldexistingrecords['error'];
            }
        } elseif (is_array($oldexistingrecords)) {
            // We have records (apparently)
            // Confirm this
            if (count($oldexistingrecords) > 0) {
                $existingrecords = $oldexistingrecords;
            } else {
                $existingrecords         = [];
                $existingrecords["info"] = "No records exist for this domain name";
            }
        } else {
            $existingrecords         = [];
            $existingrecords["info"] = "No records exist for this domain name";
        }

        // Print results in JSON and exit the code
        // This part of the code is called by AJAX
        // so we dont want to render the tempaltes, etc
        print json_encode($existingrecords);
        exit();

    } elseif (isset($_REQUEST['op']) && $_REQUEST['op'] == "deleteRecord") {

        if (isset($_REQUEST['recordid']) && strlen($_REQUEST['recordid']) > 0) {
            $emailforward['forwardID'] = $_REQUEST['recordid'];
        }

        // Set the domain name
        $emailforward['domainName'] = $params["sld"] . '.' . $params["tld"];

        // Delete the email forward
        $ret = synergywholesaledomains_APICall("deleteMailForward", $emailforward, $params);
        if ($ret["error"]) {
            $resp["error"] = "Error Deleting Email forwarder: " . $ret["error"];
        } else {
            $resp["info"] = "Email forwarder has been deleted";
        }

        // Print results in JSON and exit the code
        // This part of the code is called by AJAX
        // so we dont want to render the tempaltes, etc
        print json_encode($resp);
        exit();

    } elseif (isset($_REQUEST['op']) && $_REQUEST['op'] == "addRecord") {

        // We are adding a record, should we set the freedns name servers

        // First we need to confirm that we have the right servers and configtype
        $getns = synergywholesaledomains_APICall("getns", $request, $params);
        if (isset($getns['error'])) {
            $resp["error"] = "Unable to get the currently configured name servers";
            print json_encode($resp);
            exit();
        }

        // Loop through the returned results
        $correctNSCount = 0;
        foreach ($getns as $key => $value) {
            if (preg_match('/ns[1-3]/', $key)) {
                if (in_array($value, $freeDNSNS)) {
                    $correctNSCount++;
                }
            }
            if ($key == "dnsConfigType") {
                $dnsConfigType = $value;
            }
        }

        // See if we match all conditions required
        if ($correctNSCount != 3 || ($dnsConfigType != 2 && $dnsConfigType != 4)) {
            // We need to update the nameservers
            $newRequest['domainName']     = $params["sld"] . '.' . $params["tld"];
            $newRequest['dnsConfigType']  = 2;
            $newRequest['nameServers'][0] = "ns1.nameserver.net.au";
            $newRequest['nameServers'][1] = "ns2.nameserver.net.au";
            $newRequest['nameServers'][2] = "ns3.nameserver.net.au";
            $savens                       = synergywholesaledomains_APICall("savens", $newRequest, $params);
            if (isset($savens['error'])) {
                $resp["error"] = "Unable to set the Email forwarding Nameservers";
                print json_encode($resp);
                exit();
            }
        }

        if (isset($_REQUEST['prefix']) && strlen($_REQUEST['prefix']) > 0) {
            $emailforward['source'] = strtolower($_REQUEST['prefix']);
            if (substr($emailforward['source'], -1) == "@") {
                $emailforward['source'] = substr($emailforward['source'], 0, -1);
            }
            $domain = $params["sld"] . '.' . $params["tld"];
            if (!preg_match('/' . $domain . '$/i')) {
                $emailforward['source'] = $emailforward['source'] . "@" . $domain;
            }
        }
        if (isset($_REQUEST['forwardto']) && strlen($_REQUEST['forwardto']) > 0) {
            $emailforward['destination'] = $_REQUEST['forwardto'];
        }
        // Set the domain name
        $emailforward['domainName'] = $params["sld"] . '.' . $params["tld"];

        // Add the Mail forward
        $ret = synergywholesaledomains_APICall("addMailForward", $emailforward, $params);
        if ($ret["error"]) {
            $resp["error"] = "Error adding Mail forwarder: " . $ret["error"];
        } else {
            $resp["info"]     = "Mail forwarder has been created";
            $resp["recordID"] = $ret['recordID'];
        }

        // Print results in JSON and exit the code
        // This part of the code is called by AJAX
        // so we dont want to render the tempaltes, etc
        print json_encode($resp);
        exit();

    }

    // Return specific template details to smarty
    return [
        'templatefile' => 'domaindnsemailforwarding',
        'breadcrumb'   => [
            'clientarea.php?action=domaindetails&domainid=' . $domainid . '&modop=custom&a=manageEmailForwarding' => 'Email Forwarding',
        ],
        'vars'         => $vars,
    ];
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_custom_GetDNS($params)
{

    $request  = [];
    $errorArr = [];

    $request['domainName'] = $params["sld"] . '.' . $params["tld"];

    // Pull in any url forwards
    $ret1 = synergywholesaledomains_APICall("getURLForwards", $request, $params);
    if (is_array($ret1) && isset($ret1['error'])) {
        // Specific error occured
        $errorArr[] = 'Get URL Forwards failed: ' . $ret1['error'];
    } elseif (!is_array($ret1)) {
        // No URL Forwards present - create empty array
        $ret1 = [];
    }

    $ret2 = synergywholesaledomains_APICall("listDNSZone", $request, $params);
    if (is_array($ret2) && isset($ret2['error'])) {
        $errorArr[] = $ret2['error'];
    } elseif (is_array($ret2) && isset($ret2['error'])) {
        $errorArr[] = $ret2['error'];
    } elseif (!is_array($ret2)) {
        // No DNS Entries present - create emtpy array
        $ret2 = [];
    }

    // See if we have any errors
    if (is_array($errorArr) && count($errorArr) > 0) {
        // Build the error variable should it exist
        $errString = '';
        foreach ($errorArr as $error) {
            $errString .= $error . "<br>";
        }
        // Map it to the vars
        if (strlen($errString) > 0) {
            $vars['error'] = $errString;
        }
        // Return vars
        return $vars;
    }

    // Define dnsArr array
    $dnsArr = [];

    // Loop through each entry in ret2 and see if we have a matching url forward entry
    // If so, exclude the dns record since we don't want to show them up and have
    // customers accidentally delete them and wreck their forwards
    foreach ($ret2 as $key2 => $value2) {
        // This filter below has been commented out because url forward need to show dns records
        /*foreach ($ret1 as $key1 => $value1) {
            if ($value1['hostname'] == $value2['hostname']) {
                // Do not add this value back into the new array
                continue (2);
            }
        }*/

        // Remove the domain from the end of the record
        $value2['hostname'] = preg_replace('/(?:\.' . $params["sld"] . '.' . $params["tld"] . ').{0,1}\s{0,}$/m', '', $value2['hostname']);

        // Assign that to the primary array
        $dnsArr[] = $value2;
    }

    // Merge ret1 and the new dnsArr array together
    $newArray = array_merge($dnsArr, $ret1);

    // Return to caller
    return $newArray;

}

/*
 *    Add Client Area Custom Buttons
 */
/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_ClientAreaCustomButtonArray($params)
{

    // Register our custom pages in the ClientArea
    // These entries should always be shown regardless
    $buttonarray = [
        "Manage Child Host Records" => "manageChildHosts",
        "Domain Options"            => "domainOptions",
        "Manage DNSSEC Records"     => "manageDNSSEC",
    ];
    return $buttonarray;
}

/**
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_ClientAreaAllowedFunctions($params)
{

    // Make sure the client has enabled these particular addons before registering
    // the menu options
    $domainid = $params["domainid"];

    // Do a quick query to see if dnsmanagement or emailforwarding are enabled
    $domainInfo = Capsule::table('tbldomains')->select('dnsmanagement', 'emailforwarding')->where('id', $domainid)->get();

    $allowedArr = [];
    if ((int) $domainInfo[0]->dnsmanagement == 1) {
        $allowedArr['DNS Hosting / URL Forwarding'] = 'manageDNSURLForwarding';
    }
    if ((int) $domainInfo[0]->emailforwarding == 1) {
        $allowedArr['Email Forwarding'] = 'manageEmailForwarding';
    }
    return $allowedArr;
}

/*
 *    Perform the API calls to Synergy Wholesale API
 */
/**
 * @param  $command
 * @param  $request
 * @param  $params
 * @return mixed
 */
function synergywholesaledomains_APICall($command, $request, $params)
{

    $values                    = [];
    $soaprequest               = $request;
    $soaprequest['resellerID'] = $params["resellerID"];
    $soaprequest['apiKey']     = $params["apiKey"];

    try {
        $client = new SoapClient(null, ['location' => API_PRODUCTION, 'uri' => ""]);
        if ($command == "register") {
            $params["tld"] = end(explode(".", $params["tld"]));
            if ($params["tld"] == "au") {
                $soapresult = get_object_vars($client->domainRegisterAU($soaprequest));
            } elseif ($params["tld"] == "uk") {
                $soapresult = get_object_vars($client->domainRegisterUK($soaprequest));
            } elseif ($params["tld"] == "us") {
                $soapresult = get_object_vars($client->domainRegisterUS($soaprequest));
            } else {
                $soapresult = get_object_vars($client->domainRegister($soaprequest));
            }
        } elseif ($command == "transfer") {
            if ($params["tld"] == "uk") {
                $soapresult = get_object_vars($client->domainTransferUK($soaprequest));
            } else {
                $soapresult = get_object_vars($client->transferDomain($soaprequest));
            }
        } elseif ($command == "renew") {
            $soapresult = get_object_vars($client->renewDomain($soaprequest));
        } elseif ($command == "getns") {
            $soapresult              = get_object_vars($client->domainInfo($soaprequest));
            $values["ns1"]           = $soapresult['nameServers'][0];
            $values["ns2"]           = $soapresult['nameServers'][1];
            $values["ns3"]           = $soapresult['nameServers'][2];
            $values["ns4"]           = $soapresult['nameServers'][3];
            $values["ns5"]           = $soapresult['nameServers'][4];
            $values["dnsConfigType"] = $soapresult['dnsConfig'];
        } elseif ($command == "savens") {
            $soapresult = get_object_vars($client->updateNameServers($soaprequest));
            if ($soapresult['status'] != 'OK') {
                $values['error'] = $soapresult['errorMessage'];
            }
        } elseif ($command == "getlockstatus") {
            $soapresult = get_object_vars($client->domainInfo($soaprequest));
            if ($soapresult['domain_status'] == "clientTransferProhibited") {
                $values["lockstatus"] = "locked";
            } else {
                $values["lockstatus"] = "unlocked";
            }
        } elseif ($command == "getidprotectionstatus") {
            // Run a domainInfo command and retrieve all the details
            $soapresult = get_object_vars($client->domainInfo($soaprequest));
            // Check to see if idProtect is enabled or not
            if ($soapresult['idProtect'] == "Enabled") {
                // Enabled
                $values["idProtect"] = "Enabled";
            } else {
                // Disabled
                $values["idProtect"] = "Disabled";
            }
        } elseif ($command == "disableidprotection") {
            // Get the results of the function call
            $soapresult = get_object_vars($client->disableIDProtection($soaprequest));
        } elseif ($command == "enableidprotection") {
            // Get the results of the function call
            $soapresult = get_object_vars($client->enableIDProtection($soaprequest));
        } elseif ($command == "geteppcode") {
            // Get the results of the function call
            $soapresult = get_object_vars($client->domainInfo($soaprequest));
            // Store the epp code
            $values["eppCode"] = $soapresult['domainPassword'];
        } elseif ($command == "getcontacts") {
            // Get the results of the function call
            $soapresult = get_object_vars($client->listContacts($soaprequest));

            // Map the details from the API to WHMCS
            // Registrant should ALWAYS exist, so no need to check on that
            $values["Registrant"]["First Name"] = $soapresult['registrant']->firstname;
            $values["Registrant"]["Last Name"]  = $soapresult['registrant']->lastname;
            $values["Registrant"]["Address 1"]  = $soapresult['registrant']->address1;
            $values["Registrant"]["Address 2"]  = $soapresult['registrant']->address2;
            $values["Registrant"]["City"]       = $soapresult['registrant']->suburb;
            $values["Registrant"]["State"]      = $soapresult['registrant']->state;
            $values["Registrant"]["Postcode"]   = $soapresult['registrant']->postcode;
            $values["Registrant"]["Country"]    = $soapresult['registrant']->country;
            $values["Registrant"]["Phone"]      = $soapresult['registrant']->phone;
            $values["Registrant"]["Fax"]        = $soapresult['registrant']->fax;
            $values["Registrant"]["Email"]      = $soapresult['registrant']->email;

            // See if admin has been specified
            if (isset($soapresult['admin'])) {
                $values["Admin"]["First Name"] = $soapresult['admin']->firstname;
                $values["Admin"]["Last Name"]  = $soapresult['admin']->lastname;
                $values["Admin"]["Address 1"]  = $soapresult['admin']->address1;
                $values["Admin"]["Address 2"]  = $soapresult['admin']->address2;
                $values["Admin"]["City"]       = $soapresult['admin']->suburb;
                $values["Admin"]["State"]      = $soapresult['admin']->state;
                $values["Admin"]["Postcode"]   = $soapresult['admin']->postcode;
                $values["Admin"]["Country"]    = $soapresult['admin']->country;
                $values["Admin"]["Phone"]      = $soapresult['admin']->phone;
                $values["Admin"]["Fax"]        = $soapresult['admin']->fax;
                $values["Admin"]["Email"]      = $soapresult['admin']->email;
            }

            // See if billing has been specified
            if (isset($soapresult['billing'])) {
                $values["Billing"]["First Name"] = $soapresult['billing']->firstname;
                $values["Billing"]["Last Name"]  = $soapresult['billing']->lastname;
                $values["Billing"]["Address 1"]  = $soapresult['billing']->address1;
                $values["Billing"]["Address 2"]  = $soapresult['billing']->address2;
                $values["Billing"]["City"]       = $soapresult['billing']->suburb;
                $values["Billing"]["State"]      = $soapresult['billing']->state;
                $values["Billing"]["Postcode"]   = $soapresult['billing']->postcode;
                $values["Billing"]["Country"]    = $soapresult['billing']->country;
                $values["Billing"]["Phone"]      = $soapresult['billing']->phone;
                $values["Billing"]["Fax"]        = $soapresult['billing']->fax;
                $values["Billing"]["Email"]      = $soapresult['billing']->email;
            }

            // See if tech has been specified
            if (isset($soapresult['tech'])) {
                $values["Tech"]["First Name"] = $soapresult['tech']->firstname;
                $values["Tech"]["Last Name"]  = $soapresult['tech']->lastname;
                $values["Tech"]["Address 1"]  = $soapresult['tech']->address1;
                $values["Tech"]["Address 2"]  = $soapresult['tech']->address2;
                $values["Tech"]["City"]       = $soapresult['tech']->suburb;
                $values["Tech"]["State"]      = $soapresult['tech']->state;
                $values["Tech"]["Postcode"]   = $soapresult['tech']->postcode;
                $values["Tech"]["Country"]    = $soapresult['tech']->country;
                $values["Tech"]["Phone"]      = $soapresult['tech']->phone;
                $values["Tech"]["Fax"]        = $soapresult['tech']->fax;
                $values["Tech"]["Email"]      = $soapresult['tech']->email;
            }
        } elseif ($command == "savecontacts") {
            // Execute command
            $soapresult = get_object_vars($client->updateContact($soaprequest));
            if ($soapresult['status'] != 'OK') {
                $values['error'] = $soapresult['errorMessage'];
            }
        } elseif ($command == "addhost") {
            // Execute command
            $soapresult = get_object_vars($client->addHost($soaprequest));
        } elseif ($command == "delhost") {
            // Execute command
            $soapresult = get_object_vars($client->deleteHost($soaprequest));
        } elseif ($command == "addhostip") {
            // Execute the command
            $soapresult = get_object_vars($client->addHostIP($soaprequest));
        } elseif ($command == "delhostip") {
            // Execute the command
            $soapresult = get_object_vars($client->deleteHostIP($soaprequest));
        } elseif ($command == "sync") {

            // Execute command
            $soapresult = get_object_vars($client->domainInfo($soaprequest));

            // Return to calling function
            return $soapresult;

        } elseif ($command == "lockdomain") {
            $soapresult = get_object_vars($client->lockDomain($soaprequest));
            if ($soapresult['status'] != "OK") {
                $values = ["error" => $soapresult['errorMessage']];
            }
        } elseif ($command == "unlockdomain") {
            $soapresult = get_object_vars($client->unlockDomain($soaprequest));
            if ($soapresult['status'] != "OK") {
                $values = ["error" => $soapresult['errorMessage']];
            }
        } elseif ($command == "idprotect") {
            $soapresult = get_object_vars($client->enableIDProtection($soaprequest));
            if ($soapresult['status'] != "OK") {
                $values = ["error" => $soapresult['errorMessage']];
            }
        } elseif ($command == "idunprotect") {
            $soapresult = get_object_vars($client->disableIDProtection($soaprequest));
            if ($soapresult['status'] != "OK") {
                $values = ["error" => $soapresult['errorMessage']];
            }
        } elseif ($command == "releasedomain") {
            $soapresult = get_object_vars($client->domainReleaseUK($soaprequest));
        } elseif ($command == "listMailForwards") {
            $soapresult = get_object_vars($client->listMailForwards($soaprequest));
            if ($soapresult['status'] != "OK") {
                $values["error"] = $soapresult["error"];
            } else {
                if (is_array($soapresult['forwards'])) {
                    foreach ($soapresult['forwards'] as $forward) {
                        $forward  = get_object_vars($forward);
                        $values[] = ['prefix' => str_replace("@" . $request['domainName'], '', $forward['source']),
                            'forwardto'           => $forward['destination'], 'recordID' => $forward['id'],
                        ];
                    }
                }
            }
        } elseif ($command == "addMailForward") {
            $soapresult = get_object_vars($client->addMailForward($soaprequest));
            if ($soapresult['status'] == "OK") {
                $values['recordID'] = $soapresult['id'];
            }
        } elseif ($command == "deleteMailForward") {
            $soapresult = get_object_vars($client->deleteMailForward($soaprequest));
        } elseif ($command == "deleteDNSRecord") {
            $soapresult = get_object_vars($client->deleteDNSRecord($soaprequest));
        } elseif ($command == "addDNSRecord") {
            $soapresult = get_object_vars($client->addDNSRecord($soaprequest));
            if ($soapresult['status'] == "OK") {
                $soapresult['recordID'] = $soapresult['id'];
                unset($soapresult['id']);
                unset($soapresult['errorMessage']);
                $values = $soapresult;
            }
        } elseif ($command == "listDNSZone") {
            $soapresult = get_object_vars($client->listDNSZone($soaprequest));
            if (($soapresult['status'] == "OK") && (is_array($soapresult['records']))) {
                $values = [];
                foreach ($soapresult['records'] as $record) {
                    $recordRow = get_object_vars($record);
                    if (strtoupper($recordRow['type']) == "SOA") {
                        continue;
                    }
                    if ($recordRow['type'] == "MX" || $recordRow['type'] == "SRV") {
                        $values[] = ["hostname" => $recordRow['hostName'], "type" => $recordRow['type'], "address" => $recordRow['content'], "priority" => $recordRow['prio'], "recordid" => $recordRow['id'], "ttl" => $recordRow['ttl']];
                    } else {
                        $values[] = ["hostname" => $recordRow['hostName'], "type" => $recordRow['type'], "address" => $recordRow['content'], "recordid" => $recordRow['id'], "ttl" => $recordRow['ttl']];
                    }
                }
            }
        } elseif ($command == "getDNSRecord") {
            $soapresult = get_object_vars($client->getDNSRecord($soaprequest));
            if ($soapresult['status'] == "OK") {
                $values = $soapresult;
            }
        } elseif ($command == "dnsConfigType") {
            // added by Maks Aloksa max@modulesgarden.com
            // Run a domainInfo command and retrieve all the details
            $soapresult = get_object_vars($client->domainInfo($soaprequest));

            // retun DNS Config Type
            $values['dnsConfigType'] = $soapresult['dnsConfig'];
        } elseif ($command == "updateDnsType") {
            // Run a updateNameServers command
            $soapresult = get_object_vars($client->updateNameServers($soaprequest));
        } elseif ($command == "getICANNStatus") {
            $soapresult = get_object_vars($client->domainInfo($soaprequest));
            // retun icannStatus
            $values['icannStatus'] = $soapresult['icannStatus'];
        } elseif ($command == 'balance') {
            $soapresult        = get_object_vars($client->balanceQuery($soaprequest));
            $values['status']  = $soapresult['status'];
            $values['balance'] = $soapresult['balance'];

        } elseif ($command == 'getAllChildHosts') {

            $soapresult = get_object_vars($client->listAllHosts($soaprequest));

            if (($soapresult['status'] == "OK") && (is_array($soapresult['hosts']))) {

                $values = [];

                foreach ($soapresult['hosts'] as $hosts) {

                    $host = get_object_vars($hosts);

                    foreach ($host['ip'] as $ipRecord) {

                        $values[$host['hostName']][] = $ipRecord;

                    }
                }
            }

        } elseif ($command == 'getURLForwards') {
            $soapresult = get_object_vars($client->getSimpleURLForwards($soaprequest));

            if (($soapresult['status'] == "OK") && (is_array($soapresult['records']))) {

                $values = [];

                foreach ($soapresult['records'] as $record) {

                    $record = get_object_vars($record);

                    if ($record['redirectType'] == "P") {
                        $values[] = ["hostname" => $record['hostname'], "type" => 'URL', "address" => $record['destination'], "recordid" => $record['recordID']];
                    } elseif ($record['redirectType'] == "C") {
                        $values[] = ["hostname" => $record['hostname'], "type" => 'FRAME', "address" => $record['destination'], "recordid" => $record['recordID']];
                    }
                }
            }
        } elseif ($command == "addURLForward") {
            $soapresult = get_object_vars($client->addSimpleURLForward($soaprequest));
            if ($soapresult['status'] == "OK") {
                $soapresult['recordID'] = $soapresult['id'];
                unset($soapresult['id']);
                unset($soapresult['errorMessage']);
                $values = $soapresult;
                $soapresult['id'];
            }
        } elseif ($command == "deleteURLForward") {
            $soapresult = get_object_vars($client->deleteSimpleURLForward($soaprequest));
        } elseif ($command == "getDNSSECEntries") {
            $soapresult = get_object_vars($client->DNSSECListDS($soaprequest));
            if (($soapresult['status'] == "OK") && (is_array($soapresult['DSData']))) {
                $values = [];
                foreach ($soapresult['DSData'] as $record) {
                    $values[] = $record;
                }
            }
        } elseif ($command == "deleteDNSSECEntry") {
            $soapresult = get_object_vars($client->DNSSECRemoveDS($soaprequest));
        } elseif ($command == "addDNSSECEntry") {
            $soapresult = get_object_vars($client->DNSSECAddDS($soaprequest));
        } elseif ($command == "updateXXXMembership") {
            $soapresult = get_object_vars($client->updateXXXMembershipDetails($soaprequest));
        } elseif ($command == "resendWhoisVerif") {
            $soapresult = get_object_vars($client->resendVerificationEmail($soaprequest));
        } elseif ($command == "resendTransferApprovalEmail") {
            $soapresult = get_object_vars($client->resendTransferEmail($soaprequest));
        } elseif ($command == "getWHMCSModulesVersions") {
            $soapresult = get_object_vars($client->getWHMCSModulesVersions($soaprequest));
            $values     = $soapresult['data'];
        } elseif ($command == 'domainRenewRequired') {
        	$values = $soapresult = get_object_vars($client->domainRenewRequired($soaprequest));
        }

        $not_in_status = ['OK_TRANSFER_PENDING_COMPLETION', 'OK', 'OK_PENDING_REGO', 'OK_TRANSFER_PENDING', 'OK_TRANSFER_EMAILPENDING', 'OK_TRANSFER_EMAIL', 'OK_TRANSFER_UK_PENDING', 'OK_TRANSFER_WAITING_AUTH', 'OK_PENDING_COR', 'OUTBOUND', 'PENDINGDELETE', 'REDEMPTION', 'OK_PENDING_MANUAL', 'OK_RENEW', 'OK_NO_RENEWAL'];
        if (!in_array($soapresult['status'], $not_in_status)) {
            $values["error"] = ($soapresult['errorMessage']) ? $soapresult['errorMessage'] : serialize($soapresult);
        }

        logModuleCall('synergywholesaledomains', $command, $soaprequest, $soapresult, $soapresult, [$params["resellerID"], $params["apiKey"]]);
    } catch (SoapFault $e) {
        $values["error"] = ($e->errorMessage) ? $e->status . ' - ' . $e->errorMessage : $e;
        logModuleCall('synergywholesaledomains', $command, $soaprequest, $soapresult, $e, [$params["resellerID"], $params["apiKey"]]);
    }

    return $values;
}

/**
 * @param  $phoneNumber
 * @param  $country
 * @param  $state
 * @return mixed
 */
function synergywholesaledomains_formatPhoneNumber($phoneNumber, $country, $state = '', $countryCode = null)
{

    // If the phone number is empty, then simply return
    if ($phoneNumber == '') {
        return $phoneNumber;
    }

    // Remove any white space from the number
    $phoneNumber = preg_replace('/ /', '', $phoneNumber);

    // Remove any dashes from the number
    $phoneNumber = preg_replace('/-/', '', $phoneNumber);

    // First let's see if this is valid international format
    if (preg_match('/\+61\.[2,3,4,7,8]{1}[0-9]{8}/', $phoneNumber)) {
        // Valid International (Australian) Format so simply return
        return $phoneNumber;
    }

    // Now let's see if this is a valid international phone number
    if (preg_match('/\+[0-9]{1,3}\.[0-9]*/', $phoneNumber)) {
        // Valid International (Non Australian) Format so simply return
        return $phoneNumber;
    }

    // See if we can match onto the format 61.404040404
    preg_match_all('/^61\.([2,3,4,7,8]{1})([0-9]{8})$/', $phoneNumber, $result, PREG_PATTERN_ORDER);
    if (strlen($result[1][0]) > 0 && strlen($result[2][0]) > 0) {
        // Create new phone number (formatted)
        $phoneNumber = "+61." . $result[1][0] . $result[2][0];
    }

    // Now let'e see if this is a psuedo international phone number for Australia
    preg_match_all('/^61([2,3,4,7,8]{1})([0-9]{8})$|^\+61([2,3,4,7,8]{1})([0-9]{8})$/i', $phoneNumber, $result, PREG_PATTERN_ORDER);
    if (strlen($result[1][0]) > 0 && strlen($result[2][0]) > 0) {
        // Create new phone number (formatted)
        $phoneNumber = "+61." . $result[1][0] . $result[2][0];
        // Return phone number
        return $phoneNumber;
    } elseif (strlen($result[3][0] > 0) && strlen($result[4][0]) > 0) {
        // Create new phone number (formatted)
        $phoneNumber = "+61." . $result[3][0] . $result[4][0];
        // Return phone number
        return $phoneNumber;
    }

    // If it doesn't match any of those, it might be in AU specific phone format
    preg_match_all('/^\(0([2,3,7,8])\)([0-9]{8})|^0([2,3,4,7,8])([0-9]{8})/', $phoneNumber, $result, PREG_PATTERN_ORDER);
    if (strlen($result[1][0]) > 0 && strlen($result[2][0]) > 0) {
        // Create the new formatted phone number
        $phoneNumber = "+61." . $result[1][0] . $result[2][0];
        return $phoneNumber;
    } elseif (strlen($result[3][0] > 0) && strlen($result[4][0]) > 0) {
        // Create the new formatted phone number
        $phoneNumber = "+61." . $result[3][0] . $result[4][0];
        return $phoneNumber;
    }

    // Final check before we give up, see if the phone number is only 8 digits long
    if (preg_match('/^([0-9]{8})$/', $phoneNumber, $regs)) {
        $result = $regs[0];

        // If country is AU
        if (strtoupper($country) == 'AU') {

            // Strip any spaces from the state name and change to all UPPER case chars
            $state = preg_replace('/ /', '', $state);
            $state = strtoupper($state);

            // Switch statement on the state
            switch (strtoupper($state)) {

                case "VICTORIA":
                case "VIC":
                case "TASMANIA":
                case "TAS":
                    $areacode = "3";
                    break;

                case "QUEENSLAND":
                case "QLD":
                    $areacode = "7";
                    break;

                case "AUSTRALIANCAPITALTERRITORY":
                case "AUSTRALIACAPITALTERRITORY":
                case "ACT":
                case "NEWSOUTHWALES":
                case "NSW":
                    $areacode = "2";
                    break;

                case "SOUTHAUSTRALIA":
                case "SA":
                case "NORTHERNTERRITORY":
                case "NT":
                case "WESTERNAUSTRALIA":
                case "WA":
                    $areacode = "8";
                    break;
            }
        } else {
            // Not Australia, so simply return
            return $phoneNumber;
        }

        // Format the phone number
        $phoneNumber = "+61." . $areacode . $result;

        // Return the formatted phone number
        return $phoneNumber;
    }

    // If we get here we have no idea what type of number has been supplied
    // Simply return it
    if (is_null($countryCode)) {
        return $phoneNumber;
    } else {
        // Country code can be inserted as integer because it should never exceed three digits.
        // Phone number could potentially be bigger than PHP_MAX_INT so let's insert it 
        // into the string as a string to be ont he safe side.
        return sprintf('+%d.%s', $countryCode, $phoneNumber);
    }
}

/*
This function will validate the supplied country code
 */

/**
 * @param  $country
 * @return mixed
 */
function synergywholesaledomains_validateCountry($country)
{

    // Uppercase the country entry
    $country = strtoupper($country);

    // Set a list of valid country codes
    $cc = "AF,AX,AL,DZ,AS,AD,AO,AI,AQ,AG,AR,AM,AW,AU,AT,AZ,BS,BH,BD,BB,BY,BE,BZ,BJ,BM,BT,BO,BQ,BA,BW,BV,BR,IO,BN,BG,BF,BI,
            KH,CM,CA,CV,KY,CF,TD,CL,CN,CX,CC,CO,KM,CG,CD,CK,CR,CI,HR,CU,CW,CY,CZ,DK,DJ,DM,DO,EC,EG,SV,GQ,ER,EE,ET,FK,FO,FJ,FI,FR,
            GF,PF,TF,GA,GM,GE,DE,GH,GI,GR,GL,GD,GP,GU,GT,GG,GN,GW,GY,HT,HM,VA,HN,HK,HU,IS,IN,ID,IR,IQ,IE,IM,IL,IT,JM,JP,JE,JO,KZ,KE,
            KI,KP,KR,KW,KG,LA,LV,LB,LS,LR,LY,LI,LT,LU,MO,MK,MG,MW,MY,MV,ML,MT,MH,MQ,MR,MU,YT,MX,FM,MD,MC,MN,ME,MS,MA,MZ,MM,NA,NR,NP,
            NL,NC,NZ,NI,NE,NG,NU,NF,MP,NO,OM,PK,PW,PS,PA,PG,PY,PE,PH,PN,PL,PT,PR,QA,RE,RO,RU,RW,BL,SH,KN,LC,MF,PM,VC,WS,SM,ST,SA,SN,
            RS,SC,SL,SG,SX,SK,SI,SB,SO,ZA,GS,SS,ES,LK,SD,SR,SJ,SZ,SE,CH,SY,TW,TJ,TZ,TH,TL,TG,TK,TO,TT,TN,TR,TM,TC,TV,UG,UA,AE,GB,US,
            UM,UY,UZ,VU,VE,VN,VG,VI,WF,EH,YE,ZM,ZW";

    // Explode into an array
    $ccArray = explode(",", $cc);

    // Perform search
    $needle = array_search($country, $ccArray);

    // See if needle is FALSE or contains a key
    if ($needle === false) {
        return false;
    } else {
        return $ccArray[$needle];
    }
}

/*

This function will return a valid au state name

 */

/**
 * @param $state
 */
function synergywholesaledomains_validateAUState($state)
{

    // Trim the state name to get rid of any nasties
    $state = trim($state);

    // Remove any spaces from the state
    $state = preg_replace('/ /', '', $state);

    // Remove any dots from the state
    $state = preg_replace('/\./', '', $state);

    // Uppercase the state name
    $state = strtoupper($state);

    switch ($state) {

        case "VICTORIA":
        case "VIC":
            return "VIC";
            break;

        case "NEWSOUTHWALES":
        case "NSW":
            return "NSW";
            break;

        case "QUEENSLAND":
        case "QLD":
            return "QLD";
            break;

        case "AUSTRALIANCAPITALTERRITORY":
        case "AUSTRALIACAPITALTERRITORY":
        case "ACT":
            return "ACT";
            break;

        case "SOUTHAUSTRALIA":

        case "SA":
            return "SA";
            break;

        case "WESTERNAUSTRALIA":
        case "WA":
            return "WA";
            break;

        case "NORTHERNTERRITORY":
        case "NT":
            return "NT";
            break;

        case "TASMANIA":
        case "TAS":
            return "TAS";
            break;

        default:
            return false;
            break;
    }
}

function synergywholesaledomains_AdminCustomButtonArray()
{
    return [
        'Sync' => 'sync_adhoc',
    ];
}

function synergywholesaledomains_sync_adhoc($params)
{
    try {
        $domainInfo = Capsule::table('tbldomains')
            ->where('id', $params['domainid'])
            ->first();
    } catch (\Exception $e) {
        logModuleCall('synergywholesaledomains', 'syncButton', 'Select DB', $e->getMessage());
        return ['error' => $e->getMessage()];
    }

        if ($domainInfo->status == 'Pending Transfer') {
            return synergywholesaledomains_adhocTransferSync($params, $domainInfo);
        }

    return synergywholesaledomains_adhocSync($params, $domainInfo);
}

function synergywholesaledomains_adhocTransferSync($params, $domainInfo)
{
    $response = synergywholesaledomains_TransferSync($params);
    $update = $syncMessages = [];
    if (isset($response['error'])) {
        return $response;
    }

    if ($response['failed'] && $domainInfo->status != 'Cancelled') {
        $update['status'] = 'Cancelled';
        $errorMessage = (isset($response['reason']) ? $response['reason'] : $_LANG["domaintrffailreasonunavailable"]);
    } elseif ($response['completed']) {
        $response = synergywholesaledomains_Sync($params);
        if ($response['active'] && $domainInfo->status != 'Active') {
            $update['status'] = 'Active';
            $syncMessages[] = sprintf('Status updated from %s to Active', $domainInfo->status);
            sendMessage("Domain Transfer Completed", $domainInfo->id);
        }

        if ($response['expirydate']) {
            $newBillDate = $update['expirydate'] = $response['expirydate'];
            if ($CONFIG['DomainSyncNextDueDate'] && $CONFIG['DomainSyncNextDueDateDays']) {
                $timeCalculation = strtotime($response['expirydate']) - strtotime($CONFIG['DomainSyncNextDueDateDays'] . ' days');
                $newBillDate = date('Y-m-d', $timeCalculation);
            }

            $update['nextinvoicedate'] =  $update['nextduedate'] = $newBillDate;
        }
    }

    if (!empty($update)) {
        try {
            Capsule::table('tbldomains')
                ->where('id', $params['domainid'])
                ->update(array_merge(
                    $update, [
                        'synced' => '1'
                    ]
                ));
        } catch (\Exception $e) {
            logModuleCall('synergywholesaledomains', 'adhocTransferSync', 'Update DB', $e->getMessage());
            return ['error' => 'Error updating domain; ' . $e->getMessage()];
        }
    }

    if (isset($errorMessage)) {
        return ['error' => $errorMessage];
    }

    global $domainstatus, $nextduedate, $expirydate;
    if (isset($update['status'])) {
        $domainstatus = $update['status'];
    }

    if (isset($update['nextduedate'])) {
        $nextduedate = str_replace('-', '/', $update['nextduedate']);
        $nextduedate = date('d/m/Y', strtotime($nextduedate));
    }

    if (isset($update['expirydate'])) {
        $expirydate = str_replace('-', '/', $update['expirydate']);
        $expirydate = date('d/m/Y', strtotime($expirydate));
    }

    $hookName = '';
    switch ($update['status']) {
        case 'Active':
            $hookName = 'DomainTransferCompleted';
            break;
        case 'Cancelled':
            $hookName = 'DomainTransferFailed';
            break;
    }

    if (!empty($hookName)) {
        run_hook($hookName, [
            'domainId' => $params['domainid'],
            'domain' => $params['domainname'],
            'expiryDate' => $update['expirydate'],
            'registrar' => $params['registrar']
        ]);
    }

    return [
        'message' => nl2br(empty($syncMessages) ? 
        "Domain Sync successful." : 
        "Updated;\n    - " . implode("\n    - ", $syncMessages)
    )];
}

function synergywholesaledomains_adhocSync($params, $domainInfo)
{
    $response = synergywholesaledomains_Sync($params);
    $syncMessages = $update = [];
    if (isset($response['error'])) {
        return $response;
    }

    if ($response['active'] && $domainInfo->status != 'Active') {
        $update['status'] = 'Active';
    }

    if ($response['expired'] && $domainInfo->status != 'Expired') {
        $update['status'] = 'Expired';
    }

    if ($response['cancelled'] && $domainInfo->status == 'Active') {
        $update['status'] = 'Cancelled';
    }

    if (isset($response['transferredAway']) && $response['transferredAway'] && $domainInfo->status != 'Transferred Away') {
        $update['status'] = 'Transferred Away';
    }

    if (isset($update['status'])) {
        $syncMessages[] = sprintf('Status from "%s" to "%s"', $domainInfo->status, $update['status']);
        $domainstatus = $update['status'];
    }

    if ($response['expirydate'] && $domainInfo->expirydate != $response['expirydate']) {
        $update['expirydate'] = $response['expirydate'];
        $diExpiryFormat = date('d/m/Y', strtotime($domainInfo->expirydate));
        $updateExpiryFormat = date('d/m/Y', strtotime($update['expirydate']));
        $syncMessages[] = sprintf('Expiry date from "%s" to "%s"', $diExpiryFormat, $updateExpiryFormat);
    }

    if ($response['expirydate']) {
        $newBillDate = $update['expirydate'] = $response['expirydate'];
        if ($CONFIG['DomainSyncNextDueDate'] && $CONFIG['DomainSyncNextDueDateDays']) {
            $timeCalculation = strtotime($response['expirydate']) - strtotime($CONFIG['DomainSyncNextDueDateDays'] . ' days');
            $newBillDate = date('Y-m-d', $timeCalculation);
        }

        if ($newBillDate != $domainInfo->nextinvoicedate) {
            $update['nextinvoicedate'] = $update['nextduedate'] = $newBillDate;
            $diInvoiceDateFormat = date('d/m/Y', strtotime($domainInfo->nextinvoicedate));
            $updateBillDateFormat = date('d/m/Y', strtotime($newBillDate));
            $syncMessages[] = sprintf('Next Due Date from "%s" to "%s"', $diInvoiceDateFormat, $updateBillDateFormat);
        }
    }

    if (!empty($update)) {
        try {
            Capsule::table('tbldomains')
            ->where('id', $params['domainid'])
            ->update(array_merge(
                $update, [
                    'synced' => '1',

                ]
            ));
        } catch (\Exception $e) {
            logModuleCall('synergywholesaledomains', 'adhocSync', 'Update DB', $e->getMessage());
            return ['error' => 'Error updating domain; ' . $e->getMessage()];
        }
    }

    global $domainstatus, $nextduedate, $expirydate;
    if (isset($update['status'])) {
        $domainstatus = $update['status'];
    }

    if (isset($update['nextduedate'])) {
        $nextduedate = str_replace('-', '/', $update['nextduedate']);
        $nextduedate = date('d/m/Y', strtotime($nextduedate));
    }

    if (isset($update['expirydate'])) {
        $expirydate = str_replace('-', '/', $update['expirydate']);
        $expirydate = date('d/m/Y', strtotime($expirydate));
    }

    return [
        'message' => nl2br(empty($syncMessages) ? 
        "Domain Sync successful." : 
        "Updated;\n    - " . implode("\n    - ", $syncMessages)
    )];
}